#include <SPI.h>      // загружаем библиотеку для взаимодействиями с устройствами по SPI
#include <Ethernet.h> // загружаем библиотеку для работы с сетью
#include <SD.h>       // загружаем библиотеку для работы с sd картой
#include <EEPROM.h>   // загружаем библиотеку для работы с энергонезависимой памятью
#include <DHT.h>      // загружаем библиотеку для работы с датчиками температуры
#define REQ_BUF_SZ   40  // размер буфера используется для HTTP запроса
#define DHTPIN1 2        // балкон
#define DHTPIN2 3        // кухня
#define DHTPIN3 5        // спальня
#define DHTPIN4 6        // зал
#define DHTTYPE DHT22
DHT dht1(DHTPIN1, DHTTYPE);
DHT dht2(DHTPIN2, DHTTYPE);
DHT dht3(DHTPIN3, DHTTYPE);
DHT dht4(DHTPIN4, DHTTYPE);
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };  // MAC адрес Ethernet шилда
IPAddress ip(192, 168, 1, 16); // IP адрес
EthernetServer server(80);  // порт для работы с сервером
File webFile;
char HTTP_req[REQ_BUF_SZ] = {0}; // буферный запрос HTTP хранятся в виде нулевой строки
char req_index = 0;              // индекс в буфере HTTP_req
boolean incoming = 0; // вспомогательная переменная для получение данных из адресной строки
String all = ""; // переменная в которую пишем дынные
int t0; // полученное значение температуры котла

void setup()
{
  // отключаем Ethernet чип
  pinMode(10, OUTPUT);
  digitalWrite(10, HIGH);
  Serial.begin(9600);  // для отладки
  // инициализация SD-карты
  Serial.println("Initializing SD card...");
  if (!SD.begin(4)) {
    Serial.println("ERROR - SD card initialization failed!");
    return;    // инициализировать не удалось
  }
  Serial.println("SUCCESS - SD card initialized.");
  // проверка файла index.htm
  if (!SD.exists("index.htm")) {
    Serial.println("ERROR - Can't find index.htm file!");
    return;  // не удалось найти файл index.htm
  }
  Serial.println("SUCCESS - Found index.htm file.");
  pinMode(8, OUTPUT);    // пин реле работает как выход
  digitalWrite(8, HIGH); // реле по умолчанию выключено
  Ethernet.begin(mac, ip);  // инициализация Ethernet шилда
  server.begin();           // запуск сервера
  dht1.begin();  // инициализоция   
  dht2.begin();    // датчиков
  dht3.begin();      // температуры
  dht4.begin();        // и влажности
}

void loop()
{
  EthernetClient client = server.available();  // ожидание подключения клиента

  if (client) {  // клиент подключен?
    boolean currentLineIsBlank = true;
    while (client.connected()) {
      if (client.available()) {  // client data available to read
        char c = client.read();  // read 1 byte (character) from client
        // buffer first part of HTTP request in HTTP_req array (string)
        // leave last element in array as 0 to null terminate string (REQ_BUF_SZ - 1)
        if (req_index < (REQ_BUF_SZ - 1)) {
          HTTP_req[req_index] = c;          // save HTTP request character
          req_index++;
        }
        // считываем URL с символа "?" до первого пробела
        if (incoming && c == ' ') {
          incoming = 0;
        }
        if (c == '?') {
          incoming = 1;
        }
        // Определяем что было в URL
        if (incoming == 1) {
          String str = String(c);
          all += str;
        }
        // last line of client request is blank and ends with \n
        // respond to client only after last line received
        if (c == '\n' && currentLineIsBlank) {
          // перед выводом страницы проверяем получено ли новое значение температуры
          if (all != "") {
            all = all.substring(6, 8);
            t0 = all.toInt();
            if (12 <= t0 && t0 <= 26) {
              EEPROM.write(0, t0);  // записываем в память полученное новое значение температуры
            }
          }
          // отправка стандартного http заголовка
          client.println("HTTP/1.1 200 OK");
          client.println("Content-Type: text/html");
          client.println("Connection: keep-alive");
          client.println();
          // Ajax запрос
          if (StrContains(HTTP_req, "my_temp")) {
            // получение состояния переключателя и его отправка
            GetSwitchState(client);
          }
          else {  // запрос веб-страницы
            webFile = SD.open("index.htm");    // открыть файл веб-страницы
            if (webFile) {
              while (webFile.available()) {
                client.write(webFile.read());  // отправить веб-страницу клиенту
              }
              webFile.close();
            }
          }
          // display received HTTP request on serial port
          Serial.println(HTTP_req);
          // reset buffer index and all buffer elements to 0
          req_index = 0;
          StrClear(HTTP_req, REQ_BUF_SZ);
          break;
        }
        // every line of text received from the client ends with \r\n
        if (c == '\n') {
          // last character on line of received text
          // starting new line with next character read
          currentLineIsBlank = true;
        }
        else if (c != '\r') {
          // a text character was received from client
          currentLineIsBlank = false;
        }
      } // end if (client.available())
    } // end while (client.connected())
    delay(1);      // дать время веб-браузеру, чтобы получить данные
    client.stop(); // закрыть соединение
  } // end if (client)
  float t3 = dht3.readTemperature() - 0.3;
  if (t3 < EEPROM.read(0) - 0.2) {digitalWrite(8, LOW);}  // реле включено
  if (t3 > EEPROM.read(0) + 0.2) {digitalWrite(8, HIGH);} // реле выключено
}
// отправить состояние реле и всех показаний в веб-браузер
void GetSwitchState(EthernetClient cl)
{
  if (digitalRead(8)) {
    cl.print("0");
  }
  else {
    cl.print("1");
  }
  float AllTemp[9] = {dht1.readHumidity(), dht1.readTemperature()-0.3, dht2.readHumidity(), dht2.readTemperature()+0.4, dht3.readHumidity(), dht3.readTemperature()-0.3, dht4.readHumidity(), dht4.readTemperature()+0.2, EEPROM.read(0)};
  for (int i = 0; i < 9; i++) {
    cl.print(",");
    cl.print(AllTemp[i]);
  }
}
// sets every element of str to 0 (очищает массив)
void StrClear(char *str, char length)
{
  for (int i = 0; i < length; i++) {
    str[i] = 0;
  }
}
// searches for the string sfind in the string str
// returns 1 if string found
// returns 0 if string not found
char StrContains(char *str, char *sfind)
{
  char found = 0;
  char index = 0;
  char len;
  len = strlen(str);
  if (strlen(sfind) > len) {
    return 0;
  }
  while (index < len) {
    if (str[index] == sfind[found]) {
      found++;
      if (strlen(sfind) == found) {
        return 1;
      }
    }
    else {
      found = 0;
    }
    index++;
  }
  return 0;
}
